;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; shorthand/$.sld - Haskell-like $ operator for R7RS Scheme.
;;;
;;; This software is written by Evan Hanson <evhan@foldling.org> and
;;; placed in the Public Domain. All warranties are disclaimed.
;;;

(define-library (foldling shorthand $)
  (import (scheme base))
  (export $)
  (begin

    ;; ((f a) (g b) (h c)) => (f a (g b (h c)))
    (define-syntax $-apply
      (syntax-rules ()
        ((_ s) s)
        ((_ (s0 ...) . s)
         (s0 ... ($-apply . s)))))

    ;; ($ f a $ g b $ h c) => ((f a) (g b) (h c))
    (define-syntax $-extract
      (syntax-rules ($)
        (($ () (a0 ...) (s0 ...))
         ($-apply s0 ... (a0 ...)))
        (($ ($ ...) a s)
         (syntax-error '$ "form must not end with $"))
        (($ ($ e0 e1 ...) a (s0 ...))
         ($-extract (e1 ...) (e0) (s0 ... a)))
        (($ (e0 e1 ...) (a0 ...) s)
         ($-extract (e1 ...) (a0 ... e0) s))))

    ;; Entry point.
    (define-syntax $
      (syntax-rules ($)
        (($)
         (syntax-error '$ "at least one application must be given"))
        (($ f ...)
         ($-extract (f ...) () ()))))))
